@@ -2,7 +2,7 @@ package Geo::TCX::Trackpoint;
 use strict;
 use warnings;
 
-our $VERSION = '1.00';
+our $VERSION = '1.01';
 
 =encoding utf-8
 
@@ -107,7 +107,7 @@ Methods with respect to certain fields can be autoloaded and return the current
 
 For Basic trackpoints, LatitudeDegrees and LongitudeDegrees are the supported fields.
 
-For Full trackpoints, supported fields are: LatitudeDegrees, LongitudeDegrees, AltitudeMeters, DistanceMeters, Time, HeartRateBpm, Cadence, and SensorState.
+For Full trackpoints, supported fields are: LatitudeDegrees, LongitudeDegrees, AltitudeMeters, DistanceMeters, HeartRateBpm, Cadence, and SensorState.
 
 Some fields may contain a value of 0. It is safer to check if a field is defined with C<< if (defined $trackpoint->Cadence) >> rather than C<< if ($trackpoint->Cadence) >>.
 
@@ -256,7 +256,7 @@ use warnings;
 use DateTime::Format::ISO8601;
 use Carp qw(confess croak cluck);
 
-our $VERSION = '1.00';
+our $VERSION = '1.01';
 our @ISA=qw(Geo::TCX::Trackpoint);
 
 
@@ -311,7 +311,7 @@ sub new {
         $pt->{$1} = $2
     }
 
-    # for debugging -- allow trakcpoints with only coordinates but inspect them in debugger
+    # for debugging -- allow trackpoints with only coordinates but inspect them in debugger
     $pt->{_noTime} = 1 unless defined $pt->{Time};
     $pt->{_noDist} = 1 unless defined $pt->{DistanceMeters};
     if ($pt->{_noTime} or $pt->{_noDist}) {
@@ -319,10 +319,10 @@ sub new {
         #        $DB::single=1
     }
 
-    $pt->_set_distance( $pt->{DistanceMeters}, $previous_pt ) unless $pt->{_noDist};
+    $pt->_reset_distance( $pt->{DistanceMeters}, $previous_pt ) unless $pt->{_noDist};
     unless ($pt->{_noTime}) {
         my $orig_time_string = $pt->{Time};
-        $pt->_set_time( $pt->{Time}, $previous_pt ) unless $pt->{_noTime};
+        $pt->_reset_time( $pt->{Time}, $previous_pt ) unless $pt->{_noTime};
         print "strange ISO time not equal to time string from TCX file for this trackpoint\n"
                             if $orig_time_string ne $pt->{_time_iso8601};
     }
@@ -341,57 +341,18 @@ sub AUTOLOAD {
 
 =head2 Object Methods for class Geo::TXC::Trackpoint::Full
 
-=cut
-
-=over 4
-
-=item distance( $meters, Trackpoint object )
-
-Expects a decimal-number or integer and sets the C<DistanceMeters> field for the trackpoint, returning that value.
-
-If the elapsed-distance information for the point is not already set and another trackpoint object is also provided (e.g. the previous trackpoint), the method will compute and store the distance (in meters) from that previous point. This field is accessible via the C<distance_elapsed()> method.
-
-To simply get the C<DistanceMeters> field, call the AUTOLOAD accessor for it.
-
-=back
-
-=cut
-
-sub distance {
-    my ($pt, $meters, $previous_pt, $meters_formatted) = shift;
-    unless (@_) { return $pt->DistanceMeters };
-
-    $previous_pt = pop if ref $_[-1] and $_[-1]->isa('Geo::TCX::Trackpoint');
-    $meters = shift;
-
-    $meters_formatted  = sprintf("%.3f", $meters) if defined $meters;
-    $pt->{DistanceMeters} = $meters_formatted;
-
-    # keep this key inviolate after it is first instantiated -- similar to time()
-    # see method distance_elapsed( force => 1 ) if there is a need to change that field
-    if ( ! exists $pt->{_distance_elapsed} ) {
-        if ( $previous_pt ) {
-            my $dist_elapsed = $pt->distance - $previous_pt->distance;
-            $pt->{_distance_elapsed} = sprintf("%.3f", $dist_elapsed)
-        } else { $pt->{_distance_elapsed} = $meters_formatted }
-    }
-    return $pt->DistanceMeters
-}
-
 =over 4
 
 =item distance_elapsed( $value, force => true/false )
 
-Returns the elapsed distance (in meters) of a point as initially computed when the trackpoint was created.
+Returns the elapsed distance (in meters) of a point as initially computed when the trackpoint was created. The value is never reset unless C<< force => 1 >> is specified.
 
-For the moment, the value is never reset unless called with option C<< force => 1 >> which is not recommended.
+C<force> is needed internally by L<Geo::TCX::Lap>'s C<split()> and L<Geo::TCX::Track>'s <merge()> methods. Use with caution.
 
 =back
 
 =cut
 
-# actually the undocumented _set_distance() can be called if needed. Proceed with caution.
-
 sub distance_elapsed {
     my ($pt, $value)  = (shift, shift);
     my %opts = @_;
@@ -404,64 +365,43 @@ sub distance_elapsed {
 
 =over 4
 
-=item time( $DateTime or $time_string, Trackpoint object )
+=item Time()
 
-Expects a L<DateTime> object or a I<$time_string> in a format parseable by L<DateTime::Format::ISO8601>'s C<parse_datetime> constructor and sets the time-related fields for the trackpoint. Returns a L<DateTime> object corresponding to the time of that point.
-
-If the elapsed-time information for the point is not already set and another trackpoint object is also provided (e.g. the previous trackpoint), the method will compute and store the time (in seconds) since the timestamp of that previous point. This field is accessible via the C<time_elapsed()> method.
-
-The method maybe called without arguments to obtain an instance of a L<DateTime> object corresponding to the time of that point (no reference to DateTime objects are ever kept in trackpoints, they are too voluminous). 
-
-To simply get the C<Time> field, call the AUTOLOAD accessor for it.
+Returns the C<Time> field of a trackpoint.
 
 =back
 
 =cut
 
-sub time {
-    my ($pt, $time, $previous_pt, $dt) = shift;
-    unless (@_) { return DateTime::Format::ISO8601->parse_datetime( $pt->Time ) }
-    # NB: DateTime objects are not retained in a trackpoint given how voluminous they are
-    # this would also pose a challenge for debugging, so we provide a way to regenerate
-    # one here as above if needed
-
-    $previous_pt = pop if ref $_[-1] and $_[-1]->isa('Geo::TCX::Trackpoint');
-    $time = shift;
-
-    if ( ref( $time ) and $time->isa('DateTime') ) {
-        $dt = $time
-    } else {
-        $dt = DateTime::Format::ISO8601->parse_datetime( $time )
-    }
-
-    $pt->{Time}          = _time_format($dt);
-    $pt->{_time_iso8601} = _time_format($dt);
-    $pt->{_time_local}   = _time_format($dt, local => 1);
-    $pt->{_time_epoch}   = $dt->epoch;
-
-    # keep this key inviolate after it is first instantiated (at least, that's the current implemtation for now)
-    # elapsed time is an incremental value. If we move the time of a trackpoint with time_add, we will not change the elapsed time.
-    # here: we don't change it even if it's undef and the key exists (i.e. means we've been here once, that's enough)
-    # see method time_elapsed( force => 1 ) if there is a need to change that field (as Laps.pm does).
-    if ( ! exists $pt->{_time_elapsed} ) {
-        if ( $previous_pt ) {
-            $pt->{_time_elapsed} = $pt->time_epoch - $previous_pt->time_epoch
-        } else { $pt->{_time_elapsed} = undef }
-    }
-    return $dt
-}
+sub Time { return shift->{Time} }
 
 =over 4
 
-=item localtime( $trackpoint )
+=item time_dt ()
 
-Returns the formatted local time of the trackpoint (in the locale of the system, not that of the track per se, i.e. it is not possible to know the zone of where the track was recorded at this stage).
+=item time_datetime ()
+
+Return a L<DateTime> object corresponding to the time of a trackpoint.
 
 =back
 
 =cut
 
-sub localtime { return shift->{_time_local} }
+sub time_dt          { return DateTime::Format::ISO8601->parse_datetime( shift->Time ) }
+sub time_datetime    { return DateTime::Format::ISO8601->parse_datetime( shift->Time ) }
+# we never store a DateTime object but provide a method to create one
+
+=over 4
+
+=item time_local( $trackpoint )
+
+Returns the formatted local time of the trackpoint. The local time is always represented based on the locale of the system that calls this method, not that of where the trackpoint was recorded. It is not possible to know in which time zone a trackpoint was recorded at this stage.
+
+=back
+
+=cut
+
+sub time_local { return shift->{_time_local} }
 
 =over 4
 
@@ -469,7 +409,7 @@ sub localtime { return shift->{_time_local} }
 
 =item time_subtract( @duration )
 
-Perform L<DateTime> math on the timestamps of each lap's starttime and trackpoint by adding the specified time duration.
+Perform L<DateTime> math on the timestamps of each lap's starttime and trackpoint by adding the specified time duration and return true.
 
 The duration can be provided as an actual L<DateTime::Duration> object or an array of arguments as per the syntax of L<DateTime>'s C<add()> or C<subtract()> methods, which expect a hash of keys such as
     years        => 3,
@@ -484,8 +424,6 @@ The duration can be provided as an actual L<DateTime::Duration> object or an arr
 
 where only the relevant keys need to be specified i.e. C<< time_add( minutes > 30, seconds > 15) >>.
 
-Return the resulting L<DateTime> object.
-
 =back
 
 =cut
@@ -495,10 +433,10 @@ sub time_add {
     if (ref $_[0] and $_[0]->isa('DateTime::Duration') ) {
         $dur = shift
     } else { $dur = DateTime::Duration->new( @_ ) }
-    my $dt = $pt->time;
+    my $dt = $pt->time_datetime;
     $dt->add( $dur );
-    $pt->time( $dt );       # resets other time fields that need to be changed
-    return $dt
+    $pt->_set_time_keys( $dt );
+    return 1
 }
 
 sub time_subtract {
@@ -506,45 +444,36 @@ sub time_subtract {
     if (ref $_[0] and $_[0]->isa('DateTime::Duration') ) {
         $dur = shift
     } else { $dur = DateTime::Duration->new( @_ ) }
-    my $dt = $pt->time;
+    my $dt = $pt->time_datetime;
     $dt->subtract( $dur );
-    $pt->time( $dt );       # resets other time fields that need to be changed
-    return $dt
+    $pt->_set_time_keys( $dt );
+    return 1
 }
 
 =over 4
 
-=item time_epoch( $epoch_time )
+=item time_epoch()
 
-Sets/gets the time of a point based on an epoch time. Returns the point's epoch time.
+Returns the epoch time of a point.
 
 =back
 
 =cut
 
-sub time_epoch {
-    my ($pt, $epoch) = @_;
-    if ($epoch) {
-        my $dt = DateTime->from_epoch( epoch => $epoch );
-        $pt->time( $dt )
-    }
-    return $pt->{_time_epoch}
-}
+sub time_epoch { return shift->{_time_epoch} }
 
 =over 4
 
 =item time_elapsed( $value, force => true/false )
 
-Returns the elapsed time of a point as initially computed when the trackpoint was created.
+Returns the elapsed time of a point as initially computed when the trackpoint was created. The value is never reset unless C<< force => 1 >> is specified.
 
-For the moment, the value is never reset unless called with option C<< force => 1 >> which is not recommended.
+C<force> is needed internally by L<Geo::TCX::Lap>'s constructor, C<split()>, and C<reverse()> methods as well as L<Geo::TCX::Track>'s <reverse()>. Use with caution.
 
 =back
 
 =cut
 
-# actually the undocumented _set_time() can be called if needed. Proceed with caution.
-
 sub time_elapsed {
     my ($pt, $value)  = (shift, shift);
     my %opts = @_;
@@ -557,7 +486,7 @@ sub time_elapsed {
 
 =over 4
 
-=item time_duration( $datetime )
+=item time_duration( $datetime or $trackpoint or $string or $integer )
 
 Returns a L<DateTime::Duration> object containing the duration between the timestamps of two trackpoints. Consistent with the documentation for L<DateTime::Duration> the "duration is relative to the object from which I<$datetime> is subtracted". The duration will be positive if the timestamp of I<$datetime> occurs prior to the trackpoint, otherwise it will be negative.
 
@@ -579,14 +508,14 @@ sub time_duration {
         } else {
             croak 'object as argument must be either a DateTime or a Trackpoint instance'
                      unless $_[0]->isa('Geo::TCX::Trackpoint');
-            $datetime = $_[0]->time
+            $datetime = $_[0]->time_datetime
         }
     } elsif ($_[0] =~ /^(\d+)$/) {
         $datetime = DateTime->from_epoch( epoch => $1 )
     } else {
         $datetime = DateTime::Format::ISO8601->parse_datetime( $_[0] )
     }
-    $dt = $self->time;
+    $dt = $self->time_datetime;
 
     my $dur = $dt->subtract_datetime( $datetime );
     return $dur
@@ -624,8 +553,65 @@ sub xml_string {
     return $str
 }
 
-#
-# internal function
+# Internal methods and functions
+
+sub _reset_time {                              # called by new() and by Track.pm
+    my ($pt, $time, $previous_pt) = @_;
+    $previous_pt = pop if ref $_[-1] and $_[-1]->isa('Geo::TCX::Trackpoint');
+    delete $pt->{_time_elapsed};               # by design, immutable in _set_*
+    $pt->_set_time_keys($time, $previous_pt);
+    return 1
+}
+
+sub _reset_time_from_epoch {                   # called by Track.pm
+    my ($pt, $epoch, $previous_pt) = @_;
+    my $dt = DateTime->from_epoch( epoch => $epoch );
+    delete $pt->{_time_elapsed};
+    $pt->_set_time_keys( $dt, $previous_pt );
+    return 1
+}
+
+sub _reset_distance {                          # called by new() and by Track.pm
+    my ($pt, $distance, $previous_pt) = @_;
+    if (ref $previous_pt) {
+        croak 'second argument must be a Trackpoint object' unless $previous_pt->isa('Geo::TCX::Trackpoint')
+    }
+    delete $pt->{_distance_elapsed};
+    $pt->_set_distance_keys($distance, $previous_pt);
+    return 1
+}
+
+# Expects a I<$time_string> in a format parseable by L<DateTime::Format::ISO8601>'s C<parse_datetime> constructor
+# . sets the time-related fields for the trackpoint. Returns true.
+# . if the _time_elapsed key for the point is not already defined and another trackpoint object is also provided,
+#     e.g. the previous trackpoint, it will also set it (as number of seconds since the timestamp of that previous point)
+# . allows a DateTime obj as argument instead of $time which is required by methods that need to modify time so
+#     that we can update the keys to be consistent with the new time e.g. time_add(), time_subtract(), _reset_time_from_epoch()
+
+sub _set_time_keys {
+    my ($pt, $time, $previous_pt) = (shift, shift);
+    $previous_pt = pop if ref $_[-1] and $_[-1]->isa('Geo::TCX::Trackpoint');
+
+    my $dt;
+    if ( ref( $time ) and $time->isa('DateTime') ) {
+        $dt = $time
+    } else {
+        $pt->{Time} = $time;
+        $dt = $pt->time_datetime
+    }
+
+    $pt->{Time}          = _time_format($dt);
+    $pt->{_time_iso8601} = _time_format($dt);
+    $pt->{_time_local}   = _time_format($dt, local => 1);
+    $pt->{_time_epoch}   = $dt->epoch;
+
+    if ( ! exists $pt->{_time_elapsed} ) {          # i.e. immutable here
+        if ( $previous_pt ) {
+            $pt->{_time_elapsed} = $pt->{_time_epoch} - $previous_pt->{_time_epoch}
+        } else { $pt->{_time_elapsed} = undef }
+    }
+    return 1
+}
 
 sub _time_format {
     my $dt = shift;
@@ -637,30 +623,30 @@ sub _time_format {
     } else {
         $dt->set_formatter( $formatter_xsd )
     }
-    return "$dt"
+    return $dt->stringify
 }
 
-#
-# undocumented (for now ): also called by Track.pm
+# Expects a decimal-number or integer and sets the C<DistanceMeters> field for the trackpoint and returns true
+# . if the _distance_elapsed key for the point is not already defined and another trackpoint object is also provided,
+#     e.g. the previous trackpoint, it will also set it (number of meters from that previous point)
 
-sub _set_time {
-    my ($pt, $time, $previous_pt) = @_;
-    if (ref $previous_pt) {
-        croak 'second argument must be a Trackpoint object' unless $previous_pt->isa('Geo::TCX::Trackpoint')
-    }
-    delete $pt->{_time_elapsed};
-    $pt->time($time, $previous_pt);
-    return $pt
-}
+sub _set_distance_keys {
+    my ($pt, $meters, $previous_pt) = shift;
+    $previous_pt = pop if ref $_[-1] and $_[-1]->isa('Geo::TCX::Trackpoint');
+    $meters = shift;
 
-sub _set_distance {
-    my ($pt, $distance, $previous_pt) = @_;
-    if (ref $previous_pt) {
-        croak 'second argument must be a Trackpoint object' unless $previous_pt->isa('Geo::TCX::Trackpoint')
+    my $meters_formatted;
+    $meters_formatted  = sprintf("%.3f", $meters) if defined $meters;
+
+    $pt->{DistanceMeters} = $meters_formatted;
+
+    if ( ! exists $pt->{_distance_elapsed} ) {      # i.e. immutable here
+        if ( $previous_pt ) {
+            my $dist_elapsed = $pt->DistanceMeters - $previous_pt->DistanceMeters;
+            $pt->{_distance_elapsed} = sprintf("%.3f", $dist_elapsed)
+        } else { $pt->{_distance_elapsed} = $meters_formatted }
     }
-    delete $pt->{_distance_elapsed};
-    $pt->distance($distance, $previous_pt);
-    return $pt
+    return 1
 }
 
 }
@@ -675,7 +661,7 @@ Patrick Joly
 
 =head1 VERSION
 
-1.00
+1.01
 
 =head1 SEE ALSO
 
